"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.CALL_API = void 0;

require("whatwg-fetch");

var _humps = require("humps");

var _isEmpty = _interopRequireDefault(require("lodash/isEmpty"));

var _constants = require("./constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const getLinkRel = (response, relVal) => {
  const link = response.headers.get('link');

  if (!link) {
    return null;
  }

  const rel = link.split(',').find(s => s.indexOf(`rel="${relVal}"`) > -1);

  if (!rel) {
    return null;
  }

  return rel.split(';')[0].slice(1, -1);
};

const callApi = (url, options) => {
  return fetch(url, options).then(response => response.json().then(json => {
    if (!response.ok) {
      return Promise.reject(_objectSpread({}, json, {
        status: response.status
      }));
    }

    const camelizedJson = (0, _humps.camelizeKeys)(json);
    const next = getLinkRel(response, 'next');
    const prev = getLinkRel(response, 'prev');
    return _objectSpread({
      data: camelizedJson
    }, next && {
      next
    }, prev && {
      prev
    });
  }));
}; // Action key that carries API call info interpreted by this Redux middleware.


const CALL_API = 'CALL API'; // A Redux middleware that interprets actions with CALL_API info specified.
// Performs the call and promises when such actions are dispatched.

exports.CALL_API = CALL_API;

var _default = store => next => action => {
  const callAPI = action[CALL_API];

  if (typeof callAPI === 'undefined') {
    return next(action);
  }

  const endpoint = callAPI.endpoint,
        query = callAPI.query,
        body = callAPI.body,
        method = callAPI.method,
        token = callAPI.token,
        types = callAPI.types;

  if (typeof endpoint !== 'string') {
    throw new Error('Specify a string endpoint URL.');
  }

  if (!Array.isArray(types) || types.length !== 3) {
    throw new Error('Expected an array of three action types.');
  }

  if (!types.every(type => typeof type === 'string')) {
    throw new Error('Expected action types to be strings.');
  }

  if (query && body) {
    throw new Error('Do not call with both query and body');
  }

  if (!method) {
    throw new Error('Specify a method for api call.');
  }

  if (['GET', 'POST', 'PUT', 'DELETE'].indexOf(method) === -1) {
    throw new Error('Method must be one of {GET, POST, PUT, DELETE}');
  }

  let url = `${_constants.API_ROOT}/${endpoint}`;

  if (query && !(0, _isEmpty.default)(query)) {
    const qs = Object.keys(query).map(key => {
      const data = query[key];

      if (Array.isArray(data) && data.length > 0) {
        return data.map(val => `${key}=${encodeURIComponent(val)}`).join('&');
      } else {
        return `${key}=${encodeURIComponent(data)}`;
      }
    }).join('&');
    url = `${url}?${qs}`;
  }

  const options = {
    credentials: 'include',
    method: method
  };

  if (token) {
    options.headers.Authorization = token;
  }

  if (body) {
    if (body instanceof FormData) {
      options.body = body;
    } else {
      options.body = JSON.stringify(body);
      options.headers = {
        'Accept': 'application/json',
        'Content-Type': 'application/json'
      };
    }
  }

  const actionWith = data => {
    const finalAction = _objectSpread({}, action, data);

    delete finalAction[CALL_API];
    return finalAction;
  };

  const _types = _slicedToArray(types, 3),
        requestType = _types[0],
        successType = _types[1],
        failureType = _types[2];

  next(actionWith({
    type: requestType
  }));
  return callApi(url, options).then(response => next(actionWith({
    response,
    type: successType
  })), error => next(actionWith({
    error,
    type: failureType
  })));
};

exports.default = _default;